"""Contains calls to demonstrate dynamic detection and registration of modules"""
from types import ModuleType
from typing import List

import detector.ModuleAccessor


class WorkerDetector(object):
    def __init__(self):
        self.imported_workers: List[ModuleType] = []
        self.module_accessor = detector.ModuleAccessor.ModuleAccessor()

    def get_imported_workers(self) -> List[ModuleType]:
        return self.imported_workers

    def import_worker(self, name: str):
        try:
            worker = self.module_accessor.load_unregistered_module_by_name(name)
            self.imported_workers.append(worker)
        except ValueError:
            print(f'No module could be loaded by the name {name}')

    def trigger_message_from_imported_workers(self):
        for worker in self.imported_workers:
            worker.print_worker_message()

    def get_unregistered_workers(self) -> List[str]:
        return self.module_accessor.get_unregistered_module_names()



import pytest
from types import ModuleType
from detector.WorkerDetector import WorkerDetector
from detector.ModuleAccessor import ModuleAccessor

@pytest.fixture
def worker_detector():
    return WorkerDetector()


def test_empty_initial_workers(worker_detector):
    workers = worker_detector.get_imported_workers()
    assert isinstance(workers, list)
    assert len(workers) == 0


def test_import_new_worker(worker_detector, monkeypatch):
    def mock_module_registration(first_mockparam, second_mockparam):
        return ModuleType('mocked')

    monkeypatch.setattr(
        ModuleAccessor,
        'load_unregistered_module_by_name',
        mock_module_registration
    )
    worker_detector.import_worker('mock')
    workers = worker_detector.get_imported_workers()
    assert len(workers) > 0


def test_get_unregistered_workers_empty(worker_detector, monkeypatch):
    def mock_module_registration(mockparam):
        return []


    monkeypatch.setattr(
        ModuleAccessor,
        'get_unregistered_module_names',
        mock_module_registration
    )
    workers = worker_detector.get_unregistered_workers()
    assert isinstance(workers, list)
    assert len(workers) == 0



import pytest
from types import ModuleType
import pkg_resources

from detector.ModuleAccessor import ModuleAccessor


@pytest.fixture
def module_accessor():
    return ModuleAccessor()


class MockEntryPoint(ModuleType):
    def __init__(self, name, module_name):
        self.name = name
        self.module_name = module_name

    def load(self) -> ModuleType:
        return ModuleType('name')


def mock_entry_points(namespace):
    return [
        MockEntryPoint('mock_first', 'mock_first'),
        MockEntryPoint('mock_second', 'mock_second')
    ]


def test_detect_unregistered_entry_points(module_accessor, monkeypatch):
    monkeypatch.setattr(
        pkg_resources,
        'iter_entry_points',
        mock_entry_points
    )

    entrypoints = module_accessor._get_unregistered_module_entrypoints()
    assert isinstance(entrypoints, list)
    assert len(entrypoints) == 2
    assert isinstance(entrypoints[0], MockEntryPoint)

    names = module_accessor.get_unregistered_module_names()
    assert isinstance(names, list)
    assert len(names) == 2
    assert isinstance(names[0], str)


def test_load_incorrect_entry_point(module_accessor, monkeypatch):
    monkeypatch.setattr(
        pkg_resources,
        'iter_entry_points',
        mock_entry_points
    )
    with pytest.raises(ValueError):
        module_accessor.load_unregistered_module_by_name('')


def test_load_unregistered_entry_point(module_accessor, monkeypatch):
    monkeypatch.setattr(
        pkg_resources,
        'iter_entry_points',
        mock_entry_points
    )
    module = module_accessor.load_unregistered_module_by_name('mock_first')

    assert module is not None
    assert isinstance(module, ModuleType)



import questionary

import detector.WorkerDetector


def main():
    worker_detector = detector.WorkerDetector.WorkerDetector()

    print("Imported workers at start of run:\n{}".format(
        len(worker_detector.get_imported_workers())
    ))

    choices = worker_detector.get_unregistered_workers()
    if choices:
        workers_to_load = questionary.checkbox("All available workers:", choices=choices).ask()
        for worker in workers_to_load:
            worker_detector.import_worker(worker)
    else:
        print('Nothing to do here.')

    print("Imported workers at start of run:\n{}".format(
        len(worker_detector.get_imported_workers())
    ))
    print("Triggering print method of all workers:\n")
    worker_detector.trigger_message_from_imported_workers()

if __name__ == '__main__':
    main()



"""Class providing an API for the new module detections"""
from types import ModuleType
from typing import List

import pkg_resources


class ModuleAccessor(object):
    namespace: str = 'skidward.workers'

    def __init__(self):
        pass

    def _get_unregistered_module_entrypoints(self) -> List[ModuleType]:
        modules = []

        for ep in pkg_resources.iter_entry_points(self.namespace):
            modules.append(ep)

        return modules

    def _register_module_entrypoint(self, module_name: str) -> ModuleType:
        module: ModuleType = None

        for entrypoint in self._get_unregistered_module_entrypoints():
            if entrypoint.module_name == module_name:
                module = entrypoint.load()

        if module is None:
            raise ValueError(f'No module found by name {module_name}.')

        return module

    def get_unregistered_module_names(self) -> List[str]:
        module_names = []

        for ep in self._get_unregistered_module_entrypoints():
            module_names.append(ep.module_name)

        return module_names

    def load_unregistered_module_by_name(self, name: str) -> ModuleType:
        return self._register_module_entrypoint(name)

